package cn.net.connor.designtopart.designtopart;

import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

import com.teamcenter.rac.aifrcp.AIFUtility;
import com.teamcenter.rac.util.MessageBox;

import cn.net.connor.designtopart.DesigntopartLocale;
import cn.net.connor.designtopart.designtopart.TableBuilder.RowData;
import javafx.application.Platform;
import javafx.beans.value.ObservableValue;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TableView;
import javafx.scene.control.TitledPane;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.Priority;
import javafx.scene.layout.Region;
import javafx.scene.layout.VBox;
import k.KOperation;
import k.KOperation2;
import k.ui.KFXPanel;
import k.ui.KFXPanelController;
import k.util.KUtil;

public class DesignToPartPanelController extends KFXPanelController {
	@FXML
	private AnchorPane cover;
	@FXML
	private VBox vBox;
	@FXML
	private ScrollPane scrollPane;

	private DesignToPartDialog dialog;
	private List<TableBuilder> tableBuilders;
	protected boolean flag = false;
	protected double scrollVal;

	@Override
	public void initData(KFXPanel panel) throws Exception {
		this.dialog = (DesignToPartDialog) ((DesignToPartPanel) panel).getParentDialog();
		this.tableBuilders = dialog.controller.tableBuilders;
		setProgressVisible(true);

		new KOperation(AIFUtility.getCurrentApplication(), DesigntopartLocale.getString("readdata.STATUS")) {

			@Override
			public boolean init() throws Exception {
				return true;
			}

			@Override
			public void execute() throws Exception {
				int tableBuilderCnt = tableBuilders.size();
				for (int i = 0; i < tableBuilderCnt; i++) {
					TableBuilder tableBuilder = tableBuilders.get(i);
					TableView<RowData> table = tableBuilder.build();
					if (table == null) {
						continue;
					}
					table.addEventHandler(MouseEvent.MOUSE_CLICKED, new EventHandler<MouseEvent>() {
						@Override
						public void handle(MouseEvent mouseEvent) {
							flag = true;
							scrollVal = scrollPane.getVvalue();
							scrollPane.setVvalue(scrollVal);
							System.out.println("---MOUSE_CLICKED---");
						}
					});

					Platform.runLater(() -> {
						TitledPane pane = new TitledPane(tableBuilder.title, table);
						int height = (100 + 35 * tableBuilder.fromRevList.size()) > 205
								? (100 + 35 * tableBuilder.fromRevList.size())
								: 205;
						pane.setMinHeight(170);
						pane.setMaxHeight(height);
						pane.expandedProperty().addListener((observable, wasExpanded, isExpanded) -> {
							pane.setMaxHeight(isExpanded ? height : 0);
							pane.setMinHeight(isExpanded ? 230 : Region.USE_PREF_SIZE);
						});
						pane.setAnimated(false);
						VBox.setVgrow(pane, Priority.ALWAYS);
						vBox.getChildren().add(pane);
					});
				}
				setProgressVisible(false);
			}
		}.executeModeless();

		scrollPane.vvalueProperty()
				.addListener((ObservableValue<? extends Number> ov, Number old_val, Number new_val) -> {
					if (flag) {
						scrollPane.setVvalue(scrollVal);
						flag = false;
					}
				});
	}

	public void closeBtnClick() {
		dialog.disposeDialog();
	}

	public void createBtnClick() {
		setProgressVisible(true);
		new KOperation2(AIFUtility.getCurrentApplication(), DesigntopartLocale.getString("create.STATUS")) {

			@Override
			public boolean init() throws Exception {
				try {
					checkCanCreate();
					return true;
				} catch (Exception e) {
					e.printStackTrace();
					MessageBox.post(dialog, e);
				}
				return false;
			}

			@Override
			public void execute() {
				try {
					doCreate();
				} catch (Exception e) {
					e.printStackTrace();
					MessageBox.post(dialog, e);
				}
			}

			@Override
			public void clearCache() throws Exception {
				setProgressVisible(false);
			}

		}.executeModeless();

	}

	public void checkCanCreate() throws Exception {
		for (int i = 0; i < tableBuilders.size(); i++) {
			tableBuilders.get(i).checkCanCreate();
		}
	}

	public void doCreate() throws Exception {
		StringBuilder err = new StringBuilder();
		for (int i = 0; i < tableBuilders.size(); i++) {
			TableBuilder tableBuilder = tableBuilders.get(i);
			String status = tableBuilder.title + ": " + DesigntopartLocale.getString("create.STATUS");
			try {
				tableBuilder.doCreate(status);
			} catch (Exception e) {
				e.printStackTrace();
				err.append("[" + tableBuilder.title + "]" + e.getMessage() + "\r\n");
			}
		}
		if (err.length() > 0) {
			String errTitle = DesigntopartLocale.getString("createerror.TITLE");
			String exportFileName = errTitle + KUtil.getDateStr(new Date(), "_yyyyMMdd_HHmmss") + ".txt";
			KUtil.showScrollDialogWithExport(dialog, err.toString(), errTitle, exportFileName);
		} else {
			KUtil.info(dialog, DesigntopartLocale.getString("create.SUCCESS"));
		}
	}

	public void setProgressVisible(boolean visible) {
		if (cover != null) {
			if (Platform.isFxApplicationThread()) {
				cover.setVisible(visible);
			} else {
				Platform.runLater(() -> {
					cover.setVisible(visible);
				});
			}
			try {
				TimeUnit.MILLISECONDS.sleep(20);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
}
