package com.connor.plm;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.KeyStroke;

import com.teamcenter.rac.aif.AIFDesktop;
import com.teamcenter.rac.aif.AbstractAIFDialog;

@SuppressWarnings("serial")
public class ProgressBar extends AbstractAIFDialog {

	private JLabel messageLabel;

	private JProgressBar progressBar;

	private boolean showBar = false;

	public ProgressBar(AIFDesktop aifDesktop, Dimension dim, boolean showBar) {
		super(aifDesktop, false);
		this.showBar = showBar;
		initUI(dim);
	}

	public ProgressBar(AbstractAIFDialog pd, Dimension dimension) {
		super(pd, false);
		initUI(dimension);
	}

	public ProgressBar(Dimension dim, boolean showBar) {
		super(false);
		this.showBar = showBar;
		initUI(dim);
	}

	public ProgressBar(AbstractAIFDialog pd, Dimension dimension, boolean showBar) {
		super(pd, false);
		this.showBar = showBar;
		initUI(dimension);
	}

	private void initUI(Dimension dim) {
		this.setLayout(new BorderLayout(0, 0));
		this.setUndecorated(true);
		JPanel main = new JPanel(new GridBagLayout());
		GridBagConstraints s = new GridBagConstraints(0, 0, 1, 1, 1, 0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 5, 2, 5), 0, 0);
		main.setBackground(new Color(53, 53, 53));
		if (dim != null) {
			this.setPreferredSize(dim);
		} else {
			this.setPreferredSize(new Dimension(200, 40));
		}
		messageLabel = new JLabel(" ");
		messageLabel.setForeground(Color.white);
		progressBar = new JProgressBar();
		progressBar.setForeground(Color.black);
		main.add(messageLabel, s);
		s.gridy++;
		s.insets = new Insets(0, 5, 2, 5);
		if (showBar) {
			main.add(progressBar, s);
		}
		this.add(BorderLayout.CENTER, main);
		KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
		this.getRootPane().registerKeyboardAction(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				disposeDialog();
			}
		}, stroke, JComponent.WHEN_IN_FOCUSED_WINDOW);
		MouseAdapter ma = new MouseAdapter() {
			private Point pressPoint = new Point();

			@Override
			public void mousePressed(MouseEvent e) {
				pressPoint = e.getPoint();
			}

			@Override
			public void mouseReleased(MouseEvent e) {
				pressPoint = new Point();
			}

			@Override
			public void mouseDragged(MouseEvent e) {
				Rectangle r = ProgressBar.this.getBounds();
				setLocation(r.x + (e.getX() - pressPoint.x), r.y + (e.getY() - pressPoint.y));
			}
		};
		this.addMouseListener(ma);
		this.addMouseMotionListener(ma);
	}

	public void setText(String message) {
		// System.out.println(message);
		messageLabel.setText(message);
	}

	public void setText(String txt, int percent) {
		setText(txt);
		if (percent > progressBar.getMaximum()) {
			percent = progressBar.getMaximum();
		}
		progressBar.setValue(percent);
	}

	public void cancel() {
		disposeDialog();
	}

	public void startProgress() {
		showDialog();
	}
}